<?php
/*	*********************************************************************
	*
	* Copyright (c) 2009
	* Nicolas Germain
	* http://www.fotoremix.net/
	*
	* fotoremix : plugin spip dde traitement d'images
	*
	* Ce programme est un logiciel libre distribue sous licence GNU/GPL.
	*
	*********************************************************************	*/


include_spip (__INC_FILTRES_IMAGES__);


function fotoremix($im, $filtre)
	{
	/*
	on nettoye le filtre pour plus de surete et on regarde s'il contient une option de filtre
	*/

	$filtre_ = __cleanfilter($filtre);
	
	if (count($filtre_) > 1 && !empty($filtre_[1]))
		{
		// il y a au moins 1 option de filtre
		// dans cette version, seule la premiere option est prise en compte
		
		$optiondefitre = trim($filtre_[1]);
		}
	else $optiondefitre = '';

	/*
	on initialise quelque variables
	*/
	
	$filefilter = __PLUG_PATH__ . __DIR_FILTERS__ . $filtre_[0] . __XML_EXTENSION__;
	$filterdata = array();
	$errormessage = array();
	$allisok = true;

	/*
	calcul de la limite de taille pour l'image
	car calculs gourmands en ressource
	
	a faire et programmer ici
	*/

	/*
	lecture du filtre xml a appliquer
	*/

	if (file_exists($filefilter) && function_exists('utf8_encode') && function_exists('simplexml_load_string'))
		{
		// on lit le fichier xml et on place les infos dans un tableau	
		$xmlfiltercontent = utf8_encode(html_entity_decode(file_get_contents($filefilter)));
		
		// s'il y a une option de filtre, or remplace les caracteres @ par l'option
		if (!empty($optiondefitre)) $xmlfiltercontent = str_replace (__OPTION_TAG__, $optiondefitre, $xmlfiltercontent);
		
		$filterobject = simplexml_load_string($xmlfiltercontent);
		}
	else
		{
		// le filtre n'existe pas
		$allisok = false;
		$errormessage[] = __ERR_MSG__;	// message d'erreur a preciser
		}

	/*
	execution des instructions du filtre xml
	*/

	if ($filterobject)
		{
		// le filtre contient des infos
		// on applique les instructions dans l'ordre de leur arrivee
		
		$compteurdefonctions = 0;
		
		foreach ($filterobject->actions->action as $action)
			{
			$tag_img_ready = true;	// indique qu'une balise image n'a pas encore ete lue
			
			if (function_exists($action->fonction))
				{
				// la fonction existe, on reunit les parametres
				// a commencer par la fonction a executer
				$functiontoexecute = $action->fonction . '("' . $im . '"';
				
				// puis l'image de masque ou de texture a incorporer
				// on commence par regarder si un fichier image est defini
				// et aussi a condition que l'image existe

				if (!empty($action->image) && file_exists(__PLUG_PATH__ . __DIR_IMG_PLUG__ . $action->image) && $tag_img_ready)
					{
					$functiontoexecute .= ', "' . __PLUG_PATH__ . __DIR_IMG_PLUG__ . $action->image . '"';
					$tag_img_ready = false;
					}

				else if (!empty($action->imagealeadir) && is_dir(__PLUG_PATH__ . __DIR_IMG_PLUG__ . $action->imagealeadir . '/') && $tag_img_ready)
					{
					// pas d'image predefinie
					// on prend une image au hasard dans le repertoire indique, s'il existe
					
					$imagealeadir = '';
					$deepdir = __PLUG_PATH__ . __DIR_IMG_PLUG__ . $action->imagealeadir . '/';
					$deepdir = str_replace ('//', '/', $deepdir);	// pour eviter des erreurs de syntaxes dues a la presence d'un / en trop
					
					if (is_dir($deepdir))
						{
						// le repertoire existe, on l'ouvre et on lit les fichiers d'image qu'il contient

						$filetab[] = array();
						
						foreach (glob($deepdir . "{*.png,*.jpg,*.jpeg}", GLOB_BRACE) as $filename)
							{
							// on remplit un tableau avec les fichiers d'image du repertoire
							$filetab[] = $filename;
							}
						
						// fixe un bug php de glob, la premiere valeur du tableau n'ayant bizarrement rien a voir avec un fichier
						if (file_exists($deepdir . $filetab[0])) $ndx = 0;
						else $ndx = 1;
						
						if (count($filetab) > $ndx)
							{
							// on a trouve au moins 1 fichier, on en renvoie 1 au hasard
							$imagealeadir = $filetab[rand($ndx, count($filetab)-1)];
							}
						}
					
					$tag_img_ready = false;

					// si on a constate une erreur, on renvoie l'image d'erreur plutot que rien
					if (!empty($imagealeadir)) $functiontoexecute .= ', "' . $imagealeadir . '"';
					else $functiontoexecute .= ', "' . __PLUG_PATH__ . __DIR_IMG_PLUG__ . __PATH_IMG_ERROR__ . '"';
					}

				else if (!empty($action->imagealeaset) && $tag_img_ready)
					{
					// dernier recours, il y a un ensemble de fichier d'image
					// on prend une image au hasard dans l'ensemble
					$filetab = explode (',', str_replace ('://', '', $action->imagealeaset));
					$filetab_ = array();
					$imagealeaset = '';
					
					foreach ($filetab as $afile)
						{
						// on veille a ce que chaque cellule du tableau contienne quelque chose de valide
						
						$afile = trim($afile);
						
						if (!empty($afile) && file_exists(__PLUG_PATH__ . __DIR_IMG_PLUG__ . $afile)) $filetab_[] = $afile;
						}
					
					if (count($filetab_) > 0)
						{
						// on a trouve au moins 1 fichier, on en renvoie 1 au hasard
						$imagealeaset = __PLUG_PATH__ . __DIR_IMG_PLUG__ . trim($filetab_[rand(0, count($filetab_)-1)]);
						}

					$tag_img_ready = false;

					// si on a constate une erreur, on renvoie l'image d'erreur plutot que rien
					if (!empty($imagealeaset)) $functiontoexecute .= ', "' . $imagealeaset . '"';
					else $functiontoexecute .= ', "' . __PLUG_PATH__ . __DIR_IMG_PLUG__ . __PATH_IMG_ERROR__ . '"';
					}
				
				// enfin, le reste des parametres, balises param et paramaleaset
				// on commence par repertorier toutes les balises paramaleaset
				// on procede en 2 fois pour identifier les balises dans l'ordre d'arrivee
				
				$paramaleaset_ = array();
				
				foreach($action->paramaleaset as $param)
					{
					if (!empty($param)) $paramaleaset_[] = $param;
					}
					
				// on balaye ensuite toutes les balises param et paramaleaset
				foreach($action->xpath('./param|paramaleaset') as $param)
					{
					if (!empty($param))
						{
						if (in_array($param, $paramaleaset_))
							{
							// ce parametre est un ensemble, on prend un element au hasard
							
							$param_ = explode (',', $param);
							$param = trim($param_[rand(0, count($param_)-1)]);
							}
						
						$functiontoexecute .= ', "' . $param . '"';
						}
					}
					
				$functiontoexecute = '$im = ' . $functiontoexecute . ');';
				$compteurdefonctions++;

				eval($functiontoexecute);
				}
			}
		
		// on verifie si au moins 1 fonction a ete executee
		if ($compteurdefonctions == 0) 
			{
			// le filtre n'a pas ete applique
			$allisok = false;
			$errormessage[] = __ERR_MSG__;	// message d'erreur a preciser
			}
		}
	else
		{
		// le filtre ne peut etre applique
		$allisok = false;
		$errormessage[] = __ERR_MSG__;	// message d'erreur a preciser
		}

	/*
	effacement des images intermediaires eventuelles de facon a soulager le cache
	
	a faire ici mais fausse bonne idee ?
	avantage : soulage le cache
	inconvenient : lenteurs lors des recalculs
	*/
	
	// si tout est OK, on retourne l'image filtree
	// sinon on retourne false pour aucun traitement
	if ($allisok) return $im;
	else return false;
	}
?>